use piston_window::math::*;

use super::map::Map;

pub struct Camera {
    pub position: Vec2d,
    pub height: Scalar,
    pub width: Scalar
}

impl Camera {
    pub fn new(pos: Vec2d, width: Scalar, height: Scalar) -> Self {
        Camera {
            position: pos,
            width: width,
            height: height
        }
    }

    pub fn follow(&mut self, pos: Vec2d, map: &Map) {
        let dp = sub(pos, self.position);
        let xmargin = self.width / 4.0;
        let ymargin = self.height / 4.0;

        if dp[0] > xmargin {
            self.position[0] += dp[0] - xmargin;
        }
        if dp[0] < -xmargin {
            self.position[0] += dp[0] + xmargin;
        }
        if dp[1] > ymargin {
            self.position[1] += dp[1] - ymargin;
        }
        if dp[1] < -ymargin {
            self.position[1] += dp[1] + ymargin;
        }

        if self.position[0] + self.width / 2.0 > map.width {
            self.position[0] = map.width - self.width / 2.0;
        }
        if self.position[0] - self.width / 2.0 < 0.0 {
            self.position[0] = 0.0 + self.width / 2.0;
        }
        if self.position[1] + self.height / 2.0 > map.height {
            self.position[1] = map.height - self.height / 2.0;
        }
        if self.position[1] - self.height / 2.0 < 0.0 {
            self.position[1] = 0.0 + self.height / 2.0;
        }

    }
}
